@@ -21,11 +21,11 @@
 #include <stdio.h>
 
 #include "global.h"
-
+#ifdef DEBUG
 void
 debug_print(const char *fmt, ...)
 {
-	FILE *pfile = fopen("/home/user/pidgin-maemo-docklet.log", "a");
+	FILE *pfile = fopen("/home/nix/pidgin-maemo-docklet.log", "a");
 	if (pfile) {
 		va_list va;
 
@@ -36,3 +36,4 @@ debug_print(const char *fmt, ...)
 		fclose(pfile);
 	}
 }
+#endif /* DEBUG */
