@@ -33,12 +33,13 @@
 #include "gtkblist.h"
 #include "pidginstock.h"
 #include "core.h"
-
+#include "prefs.h"
 #include "global.h"
 
 static gchar *last_sent_buffer = NULL;
 static gsize last_sent_buffer_size = 0;
 static gboolean last_sent_blink = FALSE;
+static int my_handle = 0;
 
 static DBusConnection *dbus_connection(void);
 
@@ -64,7 +65,7 @@ pidgin_status_menu_set_tooltip(gchar *str)
 
 	if (message) {
 		dbus_message_append_args(message, DBUS_TYPE_STRING, &str, DBUS_TYPE_INVALID);
-		debug_print("|-P- *** pidgin_status_menu_set_tooltip: Making dbus call\n");
+//		debug_print("|-P- *** pidgin_status_menu_set_tooltip: Making dbus call\n");
 		reply = dbus_connection_send_with_reply_and_block(connection, message, -1, NULL);
 		dbus_message_unref(message);
 	}
@@ -138,6 +139,8 @@ pidgin_status_menu_plugin_disconnect()
 		dbus_message_unref(message);
 	}
 
+	purple_prefs_disconnect_by_handle(&my_handle);
+
 	if (reply)
 		dbus_message_unref(reply);
 }
@@ -145,13 +148,13 @@ pidgin_status_menu_plugin_disconnect()
 static void
 pidgin_maemo_docklet_ui_ops_create()
 {
-	debug_print("|-P- *** pidgin_maemo_docklet_ui_ops_create\n");
+//	debug_print("|-P- *** pidgin_maemo_docklet_ui_ops_create\n");
 }
 
 static void
 pidgin_maemo_docklet_ui_ops_destroy()
 {
-	debug_print("|-P- *** pidgin_maemo_docklet_ui_ops_destroy\n");
+//	debug_print("|-P- *** pidgin_maemo_docklet_ui_ops_destroy\n");
 	pidgin_status_menu_plugin_disconnect();
 }
 
@@ -171,6 +174,31 @@ stock_icon_to_pixbuf(const gchar *stock_id, GtkIconSize icon_size)
 	return pb;
 }
 
+static void
+mute_changed_cb(const char *name, PurplePrefType type, gconstpointer val, gconstpointer null)
+{
+	DBusConnection *conn = dbus_connection();
+	gboolean is_muted = GPOINTER_TO_INT(val);
+	DBusMessage *message = dbus_message_new_method_call(
+		PIDGIN_SYSTRAY_DBUS_SERVICE_NAME, 
+		PIDGIN_SYSTRAY_DBUS_SERVICE_PATH, 
+		NULL,
+		PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_MUTED_STATUS_CHANGED);
+
+	if (message) {
+		DBusMessage *reply = NULL;
+		dbus_message_append_args(message, DBUS_TYPE_BOOLEAN, &is_muted, DBUS_TYPE_INVALID);
+		debug_print("|-P- *** mute_changed_cb: Making dbus call\n");
+		reply = dbus_connection_send_with_reply_and_block(conn, message, -1, NULL);
+		dbus_message_unref(message);
+		message = NULL;
+		if (reply) {
+			dbus_message_unref(reply);
+			reply = NULL;
+		}
+	}
+}
+
 static void
 pidgin_maemo_docklet_ui_ops_update_icon(PurpleStatusPrimitive status, gboolean connecting, gboolean pending)
 {
@@ -225,7 +253,7 @@ pidgin_maemo_docklet_ui_ops_blank_icon()
 static void
 pidgin_maemo_docklet_ui_ops_position_menu(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer null)
 {
-	debug_print("|-P- *** pidgin_maemo_docklet_ui_ops_position_menu\n");
+//	debug_print("|-P- *** pidgin_maemo_docklet_ui_ops_position_menu\n");
 }
 
 struct docklet_ui_ops pidgin_maemo_docklet_ui_ops = {
@@ -252,7 +280,7 @@ pidgin_status_menu_plugin_connect()
 
 	if (message) {
 		dbus_error_init(&error);
-		debug_print("|-P- *** pidgin_status_menu_plugin_connect: Calling \"connect\"\n");
+//		debug_print("|-P- *** pidgin_status_menu_plugin_connect: Calling \"connect\"\n");
 		reply = dbus_connection_send_with_reply_and_block(connection, message, -1, &error);
 		if (!reply)
 			debug_print("|-P- *** pidgin_status_menu_plugin_connect: Failed to get D-Bus \"connect\" reply: %s:%s\n",
@@ -268,13 +296,18 @@ pidgin_status_menu_plugin_connect()
 				error.name ? error.name : "NULL", error.message ? error.message : "NULL");
 			ret = FALSE;
 		}
-		else
-			debug_print("|-P- *** pidgin_status_menu_plugin_connect: Successfully got boolean from D-Bus reply\n");
+//		else
+//			debug_print("|-P- *** pidgin_status_menu_plugin_connect: Successfully got boolean from D-Bus reply\n");
 		dbus_error_free(&error);
 		dbus_message_unref(reply);
 	}
 
-		debug_print("|-P- *** pidgin_status_menu_plugin_connect: Returning %s\n", ret ? "TRUE" : "FALSE");
+	if (ret) {
+		purple_prefs_connect_callback(&my_handle, PIDGIN_PREFS_ROOT "/sound/mute", (PurplePrefCallback)mute_changed_cb, NULL);
+		mute_changed_cb(PIDGIN_PREFS_ROOT "/sound/mute", PURPLE_PREF_BOOLEAN, GINT_TO_POINTER(purple_prefs_get_bool(PIDGIN_PREFS_ROOT "/sound/mute")), NULL);
+	}
+
+//	debug_print("|-P- *** pidgin_status_menu_plugin_connect: Returning %s\n", ret ? "TRUE" : "FALSE");
 	return ret;
 }
 
@@ -289,7 +322,7 @@ embed_idle(gpointer null) {
 
 static void
 status_menu_lost() {
-	debug_print("|-P- *** status_menu_lost\n");
+//	debug_print("|-P- *** status_menu_lost\n");
 	pidgin_status_menu_plugin_disconnect();
 	pidgin_docklet_init();
 
@@ -302,13 +335,13 @@ status_menu_lost() {
 static void
 status_menu_found() {
 	update_last_sent_buffer(NULL, 0, FALSE);
-	debug_print("|-P- *** status_menu_found\n");
+//	debug_print("|-P- *** status_menu_found\n");
 //	if (pidgin_docklet_get_ui_ops() != &pidgin_maemo_docklet_ui_ops) {
 		docklet_ui_init(); /* sets the X11 UI ops, making sure it is not us who get disconnected */
 		pidgin_docklet_embedded(); /* Make sure pidgin calls destroy on the X11 UI ops */
 		pidgin_docklet_uninit(); /* Calls destroy on the X11 UI ops */
 //	}
-	debug_print("|-P- *** status_menu_found: Setting maemo docklet UI ops\n");
+//	debug_print("|-P- *** status_menu_found: Setting maemo docklet UI ops\n");
 	pidgin_docklet_set_ui_ops(&pidgin_maemo_docklet_ui_ops); /* Set my UI ops */
 
 	/* Need to embed idly, because at pidgin load time, though pidgin stock is already initialized, 
@@ -337,7 +370,7 @@ message_filter(DBusConnection *connection, DBusMessage *message, gpointer null)
 			dbus_connection_flush(connection);
 			dbus_message_unref(reply);
 		}
-		debug_print("|-P- *** PidginMaemoDocklet::message_filter: Calling pidgin_docklet_clicked(%d)\n", button);
+//		debug_print("|-P- *** PidginMaemoDocklet::message_filter: Calling pidgin_docklet_clicked(%d)\n", button);
 		pidgin_docklet_clicked(button);
 	}
 	else
@@ -349,7 +382,7 @@ message_filter(DBusConnection *connection, DBusMessage *message, gpointer null)
 		!g_strcmp0("NameOwnerChanged", dbus_message_get_member(message))) {
 		const char *name, *old_owner, *new_owner;
 
-		debug_print("|-P- *** PidginMaemoDocklet::message_filter: NameOwnerChanged\n");
+//		debug_print("|-P- *** PidginMaemoDocklet::message_filter: NameOwnerChanged\n");
 
 		if (dbus_message_get_args(message, NULL, 
 			DBUS_TYPE_STRING, &name,
@@ -381,7 +414,7 @@ dbus_connection()
 		dbus_connection_setup_with_g_main(conn, NULL);
 		dbus_connection_add_filter(conn, (DBusHandleMessageFunction)message_filter, NULL, NULL);
 		dbus_bus_add_match(conn, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',path='/org/freedesktop/DBus',member='NameOwnerChanged'", NULL);
-		debug_print("|-P- *** dbus_connection: My unique name is %s\n", dbus_bus_get_unique_name(conn));
+//		debug_print("|-P- *** dbus_connection: My unique name is %s\n", dbus_bus_get_unique_name(conn));
 	}
 
 	return conn;
