@@ -19,7 +19,7 @@
  */
 
 #include <signal.h>
-
+#include <math.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <dbus/dbus.h>
@@ -39,7 +39,9 @@ struct _PidginStatusMenuPlugin {
 	DBusConnection *conn;
 	GdkPixbuf *status_pb;
 	GdkPixbuf *transp_pb;
+	GdkPixbuf *menu_pb;
 	gboolean blink_is_visible;
+	gboolean is_muted;
 };
 
 struct _PidginStatusMenuPluginClass {
@@ -59,7 +61,7 @@ pidgin_maemo_docklet_clicked(PidginStatusMenuPlugin *menu_item, int button)
 		DBusMessage *reply;
 
 		dbus_message_append_args(message, DBUS_TYPE_INT32, &button, DBUS_TYPE_INVALID);
-		debug_print("|-H- *** pidgin_maemo_docklet_clicked: Calling plugin (@%s) with button = %d\n", menu_item->pidgin_plugin_dbus_id, button);
+//		debug_print("|-H- *** pidgin_maemo_docklet_clicked: Calling plugin (@%s) with button = %d\n", menu_item->pidgin_plugin_dbus_id, button);
 		reply = dbus_connection_send_with_reply_and_block(menu_item->conn, message, -1, NULL);
 		if (reply)
 			dbus_message_unref(reply);
@@ -118,13 +120,69 @@ pixbuf_from_buffer(gchar *buffer, gsize buffer_size)
 	return pb;
 }
 
+static cairo_status_t
+png_stream_writer(GArray *array, guchar *data, guint len)
+{
+	g_array_append_vals(array, data, len);
+	return CAIRO_STATUS_SUCCESS;
+}
+
+static GdkPixbuf *
+maybe_add_circle(GdkPixbuf *src, gboolean is_muted)
+{
+	if (is_muted && src) {
+		GdkPixbuf *pb = NULL;
+		double cx = (double)gdk_pixbuf_get_width(src), cy = (double)gdk_pixbuf_get_height(src);
+		cairo_surface_t *surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, (int)cx, (int)cy);
+
+		debug_print("|-H- *** maybe_add_circle: the mute is on. The surface is %p\n", surface);
+
+		if (surface) {
+			cairo_status_t write_png_status;
+			GArray *buffer = g_array_new(FALSE, TRUE, sizeof(char));
+			cairo_t *ctx = cairo_create(surface);
+
+			if (ctx) {
+				gdk_cairo_set_source_pixbuf(ctx, src, 0.0, 0.0);
+				cairo_paint(ctx);
+
+				cairo_set_source_rgba(ctx, 1.0, 0.0, 0.0, 0.75);
+				cairo_set_line_width(ctx, 5.0);
+				cairo_set_line_cap(ctx, CAIRO_LINE_CAP_ROUND);
+/*
+				cairo_arc(ctx, cx / 2.0, cy / 2.0, min / 3.0 - 2.0, 0.0, 2.0 * M_PI);
+				cairo_stroke(ctx);
+*/
+				cairo_move_to(ctx, 3.0, 3.0);
+				cairo_line_to(ctx, cx - 3.0, cy - 3.0);
+				cairo_stroke(ctx);
+				cairo_destroy(ctx);
+			}
+			write_png_status = cairo_surface_write_to_png_stream(surface, (cairo_write_func_t)png_stream_writer, buffer);
+			debug_print("|-H- *** maybe_add_circle: write-to-png status: %s\n", cairo_status_to_string(write_png_status));
+			pb = pixbuf_from_buffer(buffer->data, buffer->len);
+			g_array_free(buffer, TRUE);
+		}
+
+		if (pb)
+			return pb;
+	}
+
+	debug_print("|-H- *** maybe_add_circle: Returning copy of original pixbuf\n");
+	return gdk_pixbuf_copy(src);
+}
+
 static void
 pidgin_status_menu_item_disconnect_plugin(PidginStatusMenuPlugin *menu_item)
 {
-	debug_print("|-H- *** pidgin_status_menu_item_disconnect_plugin: disconnecting \"%s\"\n", menu_item->pidgin_plugin_dbus_id);
+//	debug_print("|-H- *** pidgin_status_menu_item_disconnect_plugin: disconnecting \"%s\"\n", menu_item->pidgin_plugin_dbus_id);
 	g_free(menu_item->pidgin_plugin_dbus_id);
 	menu_item->pidgin_plugin_dbus_id = NULL;
 	g_object_set(G_OBJECT(menu_item->image), "pixbuf", NULL, NULL);
+	if (menu_item->menu_pb) {
+		g_object_unref(menu_item->menu_pb);
+		menu_item->menu_pb = NULL;
+	}
 	gtk_widget_hide(GTK_WIDGET(menu_item));
 }
 
@@ -134,48 +192,10 @@ message_filter(DBusConnection *connection, DBusMessage *message, PidginStatusMen
 	DBusMessage *reply = NULL;
 	DBusHandlerResult handler_return = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
-	/* connect() */
-	if (DBUS_MESSAGE_TYPE_METHOD_CALL == dbus_message_get_type(message) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_NAME, dbus_message_get_destination(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_PATH, dbus_message_get_path(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_CONNECT, dbus_message_get_member(message))) {
-		gboolean method_return = FALSE;
-
-		if (!(menu_item->pidgin_plugin_dbus_id)) {
-			const char *id = dbus_message_get_sender(message);
-
-			if (id) {
-				menu_item->pidgin_plugin_dbus_id = g_strdup(id);
-				if (menu_item->pidgin_plugin_dbus_id) {
-					debug_print("|-H- *** HildonStatusMenu::message_filter: showing menu item\n");
-					gtk_widget_show(GTK_WIDGET(menu_item));
-					method_return = TRUE;
-				}
-			}
-		}
-
-		debug_print("|-H- *** HildonStatusMenu::message_filter: replying to \"connect\" with %s\n", method_return ? "TRUE" : "FALSE");
-
-		reply = dbus_message_new_method_return(message);
-		dbus_message_append_args(reply, DBUS_TYPE_BOOLEAN, &method_return, DBUS_TYPE_INVALID);
-	}
-	else
-	/* disconnect() */
-	if (DBUS_MESSAGE_TYPE_METHOD_CALL == dbus_message_get_type(message) &&
-		!g_strcmp0(menu_item->pidgin_plugin_dbus_id, dbus_message_get_sender(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_NAME, dbus_message_get_destination(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_PATH, dbus_message_get_path(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_DISCONNECT, dbus_message_get_member(message))) {
-		const char *id = dbus_message_get_sender(message);
-
-		if (!g_strcmp0(id, menu_item->pidgin_plugin_dbus_id))
-			pidgin_status_menu_item_disconnect_plugin(menu_item);
-
-		debug_print("|-H- *** HildonStatusMenu::message_filter: replying to \"disconnect\"\n");
-		reply = dbus_message_new_method_return(message);
-	}
-	else
-	/* NameOwnerChanged(name, old, new) */
+	/***
+	 * SIGNALS
+	 ***/
+	/* NameOwnerChanged(char *name, char *old, char *new) */
 	if (DBUS_MESSAGE_TYPE_SIGNAL == dbus_message_get_type(message) &&
 		!g_strcmp0("org.freedesktop.DBus", dbus_message_get_sender(message)) &&
 		!g_strcmp0("org.freedesktop.DBus", dbus_message_get_interface(message)) &&
@@ -183,7 +203,7 @@ message_filter(DBusConnection *connection, DBusMessage *message, PidginStatusMen
 		!g_strcmp0("NameOwnerChanged", dbus_message_get_member(message))) {
 		const char *name, *old_owner, *new_owner;
 
-		debug_print("|-H- *** HildonStatusMenu::message_filter: NameOwnerChanged\n");
+//		debug_print("|-H- *** HildonStatusMenu::message_filter: NameOwnerChanged\n");
 
 		if (dbus_message_get_args(message, NULL, 
 			DBUS_TYPE_STRING, &name,
@@ -196,47 +216,108 @@ message_filter(DBusConnection *connection, DBusMessage *message, PidginStatusMen
 
 		handler_return = DBUS_HANDLER_RESULT_HANDLED;
 	}
-	/* set_tooltip(string) */
 	else
-	if (DBUS_MESSAGE_TYPE_METHOD_CALL == dbus_message_get_type(message) &&
-		!g_strcmp0(menu_item->pidgin_plugin_dbus_id, dbus_message_get_sender(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_NAME, dbus_message_get_destination(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_PATH, dbus_message_get_path(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_SET_TOOLTIP, dbus_message_get_member(message))) {
-		const char *tip_text;
+	if(DBUS_MESSAGE_TYPE_METHOD_CALL == dbus_message_get_type(message) &&
+	   !g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_NAME, dbus_message_get_destination(message)) &&
+		 !g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_PATH, dbus_message_get_path(message))) {
 
-		if (dbus_message_get_args(message, NULL, DBUS_TYPE_STRING, &tip_text, DBUS_TYPE_INVALID))
-			g_object_set(G_OBJECT(menu_item->label), "label", tip_text, NULL);
+		/***
+		 * CALLS WHERE WE DON'T CHECK THE SOURCE
+		 ***/
+		/* void connect() */
+		if (!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_CONNECT, dbus_message_get_member(message))) {
+			gboolean method_return = FALSE;
 
-		debug_print("|-H- *** HildonStatusMenu::message_filter: replying to set_tooltip(\"%s\")\n", tip_text);
+			if (!(menu_item->pidgin_plugin_dbus_id)) {
+				const char *id = dbus_message_get_sender(message);
 
-		reply = dbus_message_new_method_return(message);
-	}
-	/* update_icon(png_buffer, buffer_size) */
-	else
-	if (DBUS_MESSAGE_TYPE_METHOD_CALL == dbus_message_get_type(message) &&
-		!g_strcmp0(menu_item->pidgin_plugin_dbus_id, dbus_message_get_sender(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_NAME, dbus_message_get_destination(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_PATH, dbus_message_get_path(message)) &&
-		!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_UPDATE_ICON, dbus_message_get_member(message))) {
-		gchar *buffer;
-		gsize buffer_size;
-		gboolean blink = FALSE;
+				if (id) {
+					menu_item->pidgin_plugin_dbus_id = g_strdup(id);
+					if (menu_item->pidgin_plugin_dbus_id) {
+//						debug_print("|-H- *** HildonStatusMenu::message_filter: showing menu item\n");
+						gtk_widget_show(GTK_WIDGET(menu_item));
+						method_return = TRUE;
+					}
+				}
+			}
 
-		if (dbus_message_get_args(message, NULL, DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &buffer, &buffer_size, DBUS_TYPE_BOOLEAN, &blink, DBUS_TYPE_INVALID)) {
-			GdkPixbuf *pb = pixbuf_from_buffer(buffer, buffer_size);
-			if (pb) {
-				g_object_set(G_OBJECT(menu_item->image), "pixbuf", pb, NULL);
-				g_object_set_data_full(G_OBJECT(menu_item->image), "blink-to",
-					blink ? GINT_TO_POINTER(g_timeout_add_seconds(1, (GSourceFunc)blink_image, menu_item)) : 0,
-					(GDestroyNotify)g_source_remove);
-				g_object_unref(pb);
+//			debug_print("|-H- *** HildonStatusMenu::message_filter: replying to \"connect\" with %s\n", method_return ? "TRUE" : "FALSE");
+
+			reply = dbus_message_new_method_return(message);
+			dbus_message_append_args(reply, DBUS_TYPE_BOOLEAN, &method_return, DBUS_TYPE_INVALID);
+		}
+		else
+		if (!g_strcmp0(menu_item->pidgin_plugin_dbus_id, dbus_message_get_sender(message))) {
+			/***
+			 * CALLS WHERE WE CHECK THE SOURCE
+			 ***/
+			/* void disconnect() */
+			if (!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_DISCONNECT, dbus_message_get_member(message))) {
+				const char *id = dbus_message_get_sender(message);
+
+				if (!g_strcmp0(id, menu_item->pidgin_plugin_dbus_id))
+					pidgin_status_menu_item_disconnect_plugin(menu_item);
+
+//				debug_print("|-H- *** HildonStatusMenu::message_filter: replying to \"disconnect\"\n");
+				reply = dbus_message_new_method_return(message);
+			}
+			/* void set_tooltip(char *tip) */
+			else
+			if (!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_SET_TOOLTIP, dbus_message_get_member(message))) {
+				const char *tip_text;
+
+				if (dbus_message_get_args(message, NULL, DBUS_TYPE_STRING, &tip_text, DBUS_TYPE_INVALID))
+					g_object_set(G_OBJECT(menu_item->label), "label", tip_text, NULL);
+
+//				debug_print("|-H- *** HildonStatusMenu::message_filter: replying to set_tooltip(\"%s\")\n", tip_text);
+
+				reply = dbus_message_new_method_return(message);
+			}
+			/* void update_icon(char *png_buffer, int buffer_size, gboolean blink) */
+			else
+			if (!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_UPDATE_ICON, dbus_message_get_member(message))) {
+				gchar *buffer;
+				gsize buffer_size;
+				gboolean blink = FALSE;
+
+				if (dbus_message_get_args(message, NULL, DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &buffer, &buffer_size, DBUS_TYPE_BOOLEAN, &blink, DBUS_TYPE_INVALID)) {
+					GdkPixbuf *pb = pixbuf_from_buffer(buffer, buffer_size);
+					if (pb) {
+						if (menu_item->menu_pb)
+							g_object_unref(menu_item->menu_pb);
+						menu_item->menu_pb = pb;
+						pb = maybe_add_circle(pb, menu_item->is_muted);
+					}
+					if (pb) {
+						g_object_set(G_OBJECT(menu_item->image), "pixbuf", pb, NULL);
+						g_object_set_data_full(G_OBJECT(menu_item->image), "blink-to",
+							blink ? GINT_TO_POINTER(g_timeout_add_seconds(1, (GSourceFunc)blink_image, menu_item)) : 0,
+							(GDestroyNotify)g_source_remove);
+						g_object_unref(pb);
+					}
+				}
+
+				debug_print("|-H- *** HildonStatusMenu::message_filter: replying to update_pixbuf\n");
+
+				reply = dbus_message_new_method_return(message);
+			}
+			/* void muted_status_changed(gboolean is_muted) */
+			else
+			if (!g_strcmp0(PIDGIN_SYSTRAY_DBUS_SERVICE_METHOD_MUTED_STATUS_CHANGED, dbus_message_get_member(message))) {
+				gboolean is_muted;
+
+				if (dbus_message_get_args(message, NULL, DBUS_TYPE_BOOLEAN, &is_muted, DBUS_TYPE_INVALID)) {
+					GdkPixbuf *pb = NULL;
+
+					menu_item->is_muted = is_muted;
+
+					debug_print("|-H- *** HildonStatusMenu::message_filter: maybe adding circle to menu_pb (%p)\n", menu_item->menu_pb);
+					pb = maybe_add_circle(menu_item->menu_pb, is_muted);
+					g_object_set(G_OBJECT(menu_item->image), "pixbuf", pb, NULL);
+					g_object_unref(pb);
+				}
 			}
 		}
-
-		debug_print("|-H- *** HildonStatusMenu::message_filter: replying to update_pixbuf\n");
-
-		reply = dbus_message_new_method_return(message);
 	}
 
 	if (reply) {
@@ -246,8 +327,8 @@ message_filter(DBusConnection *connection, DBusMessage *message, PidginStatusMen
 		dbus_message_unref(reply);
 		handler_return = DBUS_HANDLER_RESULT_HANDLED;
 	}
-	else
-		debug_print("|-H- *** HildonStatusMenu::message_filter: NO REPLY\n");
+//	else
+//		debug_print("|-H- *** HildonStatusMenu::message_filter: NO REPLY\n");
 	return handler_return;
 }
 
@@ -341,4 +422,8 @@ pidgin_status_menu_plugin_init(PidginStatusMenuPlugin *menu_item)
 			NULL),
 		TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(menu_item), hbox);
+
+	menu_item->status_pb = NULL;
+	menu_item->menu_pb = NULL;
+	menu_item->is_muted = FALSE;
 }
